/*******************************************************************************
 * Copyright (C) 2017 Philipp Spohr
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
package de.hhu.ba.yoshikoWrapper.help;

import java.awt.Container;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.WeakHashMap;

import org.slf4j.Logger;

import de.hhu.ba.yoshikoWrapper.logging.YoshikoLogger;
import de.hhu.ba.yoshikoWrapper.swing.components.EditCostPanel;

/**
 * Helper class that provides URIs to access the Bachelor Thesis paper sections that can be used as a manual
 *
 */
public final class HelpLinks {

	//SYMBOLIC LINKS

	/**
	 * Convenience function for better code readability
	 */
	private static final Logger logger = YoshikoLogger.getInstance().getLogger();

	//

	/**
	 * The base location of the bachelor thesis paper
	 */
	//TODO: Add final location when published
	private static final String docLocation = "https://gitlab.cs.uni-duesseldorf.de/spohr/YoshikoWrapper/tree/master";

	/**
	 * A HashMap linking Classes to a link, pointing to a specific subsection of the BachelorThesis paper that is relevant for understanding the Class
	 */
	private static final WeakHashMap <Class<? extends Container>,URI> helpLinks;

	/**
	 * Initialization for the HashMap
	 */
	static {
		helpLinks = new WeakHashMap<Class<? extends Container>,URI>();
		try {
			helpLinks.put(EditCostPanel.class, (buildURI("subsubsection.2.2.1")));
		} catch (URISyntaxException e) {
			logger.error("Error initializing URI for help file:\n"+e.getMessage());
		}
	}


	/**
	 * Returns the URI that leads to the relevant help section for the class provided
	 * @param c The Container class for which the help link should be fetched
	 * @return The help link as URI
	 */
	public static URI getHelpLink(Class<? extends Container> c) {
		if (helpLinks.containsKey(c)) {
			return helpLinks.get(c);
		}
		return null;
	}

	/**
	 * Internal helper function that builds a valid URI from the base address and the fragment denoting the relevant section
	 * @param fragment The fragment naming the nameddest in the pdf file
	 * @return The compound URI made from base address and fragment
	 * @throws URISyntaxException If invalid -> Should never be called during runtime in a stable version
	 */
	private static URI buildURI(String fragment) throws URISyntaxException {
		return new URI(null,docLocation,"nameddest="+fragment);
	}

}
